unit GameField;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics;

{$R *.res}

type
  TDrawSector = procedure(Sender: TObject;
    X, Y: Integer; ARect: TRect) of object;

type
  TGameField = class(TGraphicControl)
  private
    FLineColor: TColor;
    FLineWidth: Integer;
    FSectorsCount: Integer;
    FSectorSize: Integer;
    FOnDrawSector: TDrawSector;
    FAutoSize: Boolean;
    procedure SetLineColor(const Value: TColor);virtual;
    procedure SetLineWidth(const Value: Integer);virtual;
    procedure SetSectorsCount(const Value: Integer);virtual;
    procedure SetSectorSize(const Value: Integer);virtual;
    procedure SetAutoSize(const Value: Boolean);virtual;
    { Private declarations }
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    property Canvas;
    procedure DrawAll;
    procedure DrawOne(X,Y: Integer);
    { Public declarations }
  published
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property Color;
    property LineColor: TColor read FLineColor write SetLineColor;
    property LineWidth: Integer read FLineWidth write SetLineWidth;
    property SectorsCount: Integer read FSectorsCount write SetSectorsCount;
    property SectorSize: Integer read FSectorSize write SetSectorSize;
    property OnDrawSector: TDrawSector read FOnDrawSector write FOnDrawSector;
    property OnCanResize;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('fse', [TGameField]);
end;

{ TGameField }

constructor TGameField.Create(AOwner: TComponent);
begin
  inherited;
  FAutoSize:=true;
  FLineColor:=clNavy;
  FLineWidth:=1;
  FSectorsCount:=10;
  FSectorSize:=15;
  Width:=151;
  Height:=151;
end;

procedure TGameField.DrawAll;
var
  X, Y: Integer;
begin
  with Canvas do
  begin
    if Assigned(FOnDrawSector)then
    for Y:=0 to SectorsCount-1 do
      for X:=0 to SectorsCount-1 do
        DrawOne(X,Y);
  end;
end;

procedure TGameField.DrawOne(X, Y: Integer);
begin
    if Assigned(FOnDrawSector)then
      FOnDrawSector(Self,X,Y,Rect(
      X*SectorSize+1,Y*SectorSize+1,
      X*SectorSize+SectorSize,Y*SectorSize+SectorSize))
end;

procedure TGameField.Paint;
var
  I: Integer;
  AutoSizeValue: Integer;
begin
  inherited;
  AutoSizeValue:=FSectorsCount*FSectorSize+1;
  if AutoSize then
    if (Width<>AutoSizeValue)or(Height<>AutoSizeValue)then
    begin
      Width:=AutoSizeValue;
      Height:=AutoSizeValue;
    end;
  with Canvas do
  begin
    Brush.Color:=Color;
    Brush.Style:=bsSolid;
    Pen.Color:=LineColor;
    Pen.Mode:=pmCopy;
    Pen.Style:=psSolid;
    Pen.Width:=LineWidth;
    FillRect(ClipRect);
    for I:=0 to SectorsCount do
    begin
      MoveTo(0,I*SectorSize);
      LineTo(SectorSize*SectorsCount,I*SectorSize);
      MoveTo(I*SectorSize,0);
      LineTo(I*SectorSize,SectorSize*SectorsCount);
    end;
    DrawAll;
  end;
end;

procedure TGameField.SetAutoSize(const Value: Boolean);
begin
  FAutoSize := Value;
  Invalidate;
end;

procedure TGameField.SetLineColor(const Value: TColor);
begin
  FLineColor := Value;
  Invalidate;
end;

procedure TGameField.SetLineWidth(const Value: Integer);
begin
  FLineWidth := Value;
  Invalidate;
end;

procedure TGameField.SetSectorsCount(const Value: Integer);
begin
  FSectorsCount := Value;
  Invalidate;
end;

procedure TGameField.SetSectorSize(const Value: Integer);
begin
  FSectorSize := Value;
  Invalidate;
end;

end.
